#!/bin/sh
#
# Author:   Christian Rost (chr@baltic-online.de)
# Purpose:  Shell-Interface to rrdtool
#
RRDCNF=/etc/rrd.conf
. $RRDCNF

######################################
# init() function to create .rrd-file
# $1=VAR-PRAEFIX $2=VAR-POSTFIX
######################################
init()
{
	# Calculate optimal step by assuming highest precision on the last-xhour plot
	OPTSTEP=$(( $HOURS * 3600 / $PIXWIDTH * $H_RES ))
	[ $OPTSTEP != $STEP ] && echo "WARNING: Your optimal step is: $OPTSTEP. But you are using: $STEP"

	ARH_AVG=$(( $HOURS * 3600   / $PIXWIDTH * $H_RES / $STEP )); ARH_CNT=$(( $HOURS * 3600   / $ARH_AVG / $STEP ))
	ARD_AVG=$(( $DAYS  * 86400  / $PIXWIDTH * $D_RES / $STEP )); ARD_CNT=$(( $DAYS  * 86400  / $ARD_AVG / $STEP ))
	ARW_AVG=$(( $WEEKS * 604800 / $PIXWIDTH * $W_RES / $STEP )); ARW_CNT=$(( $WEEKS * 604800 / $ARW_AVG / $STEP ))

	PRAEFIX=$1; POSTFIX=$2
	DSOURCE=""

	echo "Init storage for $HOURS hours on averages: $ARH_AVG x $STEP seconds ($ARH_CNT total values)"
	echo "Init storage for $DAYS days on averages: $ARD_AVG x $STEP seconds ($ARD_CNT total values)"
	echo "Init storage for $WEEKS weeks on averages: $ARW_AVG x $STEP seconds ($ARW_CNT total values)"

	eval SOURCE=\$${PRAEFIX}_${POSTFIX}
	for i in $SOURCE
	do
		eval var=\$${PRAEFIX}_${i}
		IFS="|"; set -- $var; unset IFS
		[ "${1}" = "${1#+}" ] && DSOURCE="$DSOURCE DS:$(echo ${1#-}):${3}:$MAXSTEP:0:${4}"
	done
	$RRDTOOL create $RRDDIR/$PRAEFIX.rrd \
		--step ${STEP} \
		$DSOURCE \
			RRA:AVERAGE:0.5:$ARH_AVG:$ARH_CNT \
			RRA:AVERAGE:0.5:$ARD_AVG:$ARD_CNT \
			RRA:AVERAGE:0.5:$ARW_AVG:$ARW_CNT \
			RRA:MAX:0.5:$ARH_AVG:$ARH_CNT \
			RRA:MAX:0.5:$ARD_AVG:$ARD_CNT \
			RRA:MAX:0.5:$ARW_AVG:$ARW_CNT
	return $?
}

#################################################################################################################
# graph() functions to create png-graphics, def or cgi
# $1=IMG-DIR $2=VAR-PRAEFIX, $3=VAR-POSTFIX, $4=Headline, $5=end, $6=start, $7=base,
# $8=DEFS/ENTRIES $9=other rrdgraph options (Vertlabel,...)
#################################################################################################################
do_rrdgraph()
{
	# Note: $8 and $9 contain quoded stuff, but rrdtool expects properly
	# splitted input. Replaced quotes by 0xa0 == ASCII fixed space char
	$RRDTOOL graph "$1/$2_$3.png" \
		--title "$4" \
		--imgformat $PIXFORMAT \
		--width $PIXWIDTH \
		--height $PIXHEIGHT \
		--end "$5" \
		--start "$6" \
		--base "$7" \
		$(echo $8|sed ':n;s/"\([^" ]*\)"/\1/;s/\("[^"]*\) \([^"]*"\)/\1\2/;tn') \
		$(echo $9|sed ':n;s/"\([^" ]*\)"/\1/;s/\("[^"]*\) \([^"]*"\)/\1\2/;tn') \
		COMMENT:"$DATE\c"
}

do_rrddef()
{
	# Note: quoted stuff in $8 and $9 work fine with rrdcgi
	cat << EOF > $1/$2_$3.def

<RRD::GRAPH $1/<RRD::CV START>$2_$3.png \
--title "$4" \
--imgformat $PIXFORMAT \
--width $PIXWIDTH \
--height $PIXHEIGHT \
--end "$5" \
--start <RRD::CV START> \
--base $7 \
--imginfo "<IMG NAME=$2_$3 SRC=$RRDCGIIMGPATH/%s border=0 OnClick=zoomToggle('${SCGIPIXWIDTH}px','${NCGIPIXWIDTH}px',this); WIDTH=${SCGIPIXWIDTH}px>" \
--lazy \
$8 $9 COMMENT:"<RRD::TIME::NOW %c>\c">
EOF
}

do_rrdcgi()
{
	cat << EOF >> $RRDCGISCRIPT
$NEWLINE <TD valign=top>$4<br>
<RRD::INCLUDE $1/$2_$3.def>
</TD>
EOF
}

gen_cgi_header()
{
	cat << EOF > $RRDCGISCRIPT
#!$RRDCGI
<HEAD><TITLE>RRDCGI Router Statistics</TITLE></HEAD>
<SCRIPT TYPE="text/javascript" LANGUAGE="JavaScript">
function update_radio_check(robj,value) { for (var i=0; i<robj.length; i++) if (robj[i].value==value) robj[i].checked=1; }
var nW,oW;
function zoomToggle(iWideSmall,iWideLarge,whichImage){
oW=whichImage.style.width;
if(oW==iWideLarge){
nW=iWideSmall;}else{
nW=iWideLarge;}
whichImage.style.width=nW;
}
</SCRIPT>
<BODY onload="update_radio_check(document.RangeForm.START,'<RRD::CV START>')">
<H1>RRDCGI Router Statistics <font size=-2><em>(<RRD::TIME::NOW %c>)</em></font></H1>
<FORM NAME=RangeForm>
<INPUT NAME=START TYPE=RADIO VALUE=end-${HOURS}h checked>${HOURS} hours ago
<INPUT NAME=START TYPE=RADIO VALUE=end-${DAYS}d>${DAYS} days ago
<INPUT NAME=START TYPE=RADIO VALUE=end-${WEEKS}w>${WEEKS} weeks ago
<INPUT TYPE=SUBMIT>
</FORM>
<P>
<TABLE>
EOF
}

gen_cgi_footer()
{
	cat << EOF >> $RRDCGISCRIPT
</TABLE>
</P>
</BODY>
</HTML>
EOF
	chmod +x $RRDCGISCRIPT
}

#################################################################################################################
# $1=do_what $2=VAR-PRAEFIX, $3=VAR-POSTFIX, $4=Headline, $5=Vertlabel, $6=start, $7=end, $8=base, $9=other rrdgraph options
#################################################################################################################
do_graph()
{
	do_what=$1; PRAEFIX=$2; POSTFIX=$3; HEADLINE=$4; VERTLABEL=$5; START=$6; END=$7; BASE=$8; SRC=${RRDDIR}/${PRAEFIX}.rrd
	if [ ! -r "$SRC" ]
	then
		echo "$SRC not found. Aborting all operations now..."
		exit 1
	fi

	# For Graphing Memory BASE should be set to 1024, default is 1000
	[ -n "$8" ] && BASE=$8 || BASE=1000
	[ -n "$9" ] && OPTS=$9 || OPTS="--lower-limit 0"
	DEFS=""; ENTRIES=""
	eval SOURCE=\$${PRAEFIX}_${POSTFIX}
	for i in $SOURCE
	do
		eval var=\$${PRAEFIX}_${i}
		IFS="|"; set -- $var; unset IFS
		NEG=0
		VAR=$(echo ${1#[+-]})
		if [ "${1}" != "${1#-}" ]
		then
			NEG=1
			[ -n "${8}" ] && DEFS="$DEFS DEF:${VAR}=$SRC:${VAR}:AVERAGE DEF:MAX${VAR}=$SRC:${VAR}:MAX ${8} CDEF:N${VAR}C=${VAR}C,-1,* " \
			  	      || DEFS="$DEFS DEF:${VAR}=$SRC:${VAR}:AVERAGE DEF:MAX${VAR}=$SRC:${VAR}:MAX CDEF:N${VAR}=${VAR},-1,*"
		elif [ "${1}" = "${1#+}" ]
		then
			[ -n "${8}" ] && DEFS="$DEFS DEF:${VAR}=$SRC:${VAR}:AVERAGE DEF:MAX${VAR}=$SRC:${VAR}:MAX ${8}" \
				      || DEFS="$DEFS DEF:${VAR}=$SRC:${VAR}:AVERAGE DEF:MAX${VAR}=$SRC:${VAR}:MAX"
		else
			[ -n "${8}" ] && DEFS="$DEFS ${8}"
		fi

		if [ -n "${7}" ]
		then
			if [ -n "${8}" ]
			then
				[ "$NEG" = 1 ] && ENTRIES="$ENTRIES ${6}:N${VAR}C${5}:\"${2}\" GPRINT:${VAR}C:AVERAGE:Average\:${7} GPRINT:MAX${VAR}C:MAX:Max\:${7} GPRINT:${VAR}C:LAST:Last\:${7}\\j" \
					       || ENTRIES="$ENTRIES ${6}:${VAR}C${5}:\"${2}\"  GPRINT:${VAR}C:AVERAGE:Average\:${7} GPRINT:MAX${VAR}C:MAX:Max\:${7} GPRINT:${VAR}C:LAST:Last\:${7}\\j"
			else
				[ "$NEG" = 1 ] && ENTRIES="$ENTRIES ${6}:N${VAR}${5}:\"${2}\" GPRINT:${VAR}:AVERAGE:Average\:${7} GPRINT:MAX${VAR}:MAX:Max\:${7} GPRINT:${VAR}:LAST:Last\:${7}\\j" \
					       || ENTRIES="$ENTRIES ${6}:${VAR}${5}:\"${2}\"  GPRINT:${VAR}:AVERAGE:Average\:${7} GPRINT:MAX${VAR}:MAX:Max\:${7} GPRINT:${VAR}:LAST:Last\:${7}\\j"
			fi
		else
			[ "$NEG" = 1 ] && ENTRIES="$ENTRIES ${6}:N${VAR}${5}:\"${2}\" GPRINT:${VAR}:AVERAGE:Average\:%6.2lf%s GPRINT:MAX${VAR}:MAX:Max\:%6.0lf%s GPRINT:${VAR}:LAST:Last\:%6.2lf%s\\j" \
				       || ENTRIES="$ENTRIES ${6}:${VAR}${5}:\"${2}\"  GPRINT:${VAR}:AVERAGE:Average\:%6.2lf%s GPRINT:MAX${VAR}:MAX:Max\:%6.0lf%s GPRINT:${VAR}:LAST:Last\:%6.2lf%s\\j"
		fi
	done

	$do_what "$IMGDIR" "$PRAEFIX" "$POSTFIX" "$HEADLINE" "$END" "$START" "$BASE" "$DEFS $ENTRIES" "--vertical-label \"$VERTLABEL\" $OPTS"
}

# Check if all rrd-files are present
# and create them if not
init_loop()
{
	for j in $ALL_PRAEFIXES
	do
		if [ ! -f $RRDDIR/$j.rrd ]
		then
			echo "Creating $RRDDIR/$j.rrd ..."
			init $j ALL
			[ ! -f $RRDDIR/$j.rrd ] && EXIT=1
		fi
	done
	[ "$EXIT" = 1 ] && return 1 || return 0
}


graph_loop()
{
	do_what=$1; start=$2; end=$3; c=1
	while [ $c -le $GRAPH_MAX ]
	do
		eval var=\$GRAPH_${c}
		# $1=VAR-PRAEFIX $2=VAR-POSTFIX $3=Headline $4=Vertlabel $5=base $6=other rrdgraph options
		IFS="|"; set -- $var; unset IFS
		[ 1 -eq $(( $c  % 2 )) ] && NEWLINE="</TR><TR>" || NEWLINE=""
		do_graph $do_what $1 $2 "$3" "$4" $start $end $5 "$6"
		c=$(( $c + 1 ));
	done
}


case "$1" in
	init)
			if init_loop
			then
				exit 0
			else
			 	echo "Initialisation of rrd-database-files failed"
				exit 1
			fi
			;;
	def)
			echo "Updating $IMGDIR/*.def"
			graph_loop do_rrddef end now
			;;
	cgi)
			echo "Updating $RRDCGISCRIPT"
			gen_cgi_header
			# The start-time will be not used
			graph_loop do_rrdcgi end now
			gen_cgi_footer
			;;
	images)
			# Create the graphs
			[ -n "$2" ] && STH=$2 || STH=24h
			echo "Updating Graphs ($STH ago -> now) for $RRDDIR/*.rrd"
			graph_loop do_rrdgraph "end-${STH}" now
			;;
	fetch)
			# Fetch data from rrdcollect daemon
			if [ ! -n "$3" ]
			then
				echo "Usage: $0 fetch <host> <port>"
				exit 1
			else
				echo "Fetching ascii-logs from rrdcollect daemon"
				(cd $RRDDIR && nc $2 $3 | sed -n "s:^\(update .*\):$RRDTOOL \1:p" | sh -x ) 2>&1 | wc -l #>/dev/null
				exit 0
			fi
			;;
	*)		echo "Usage: $0 <init|cgi|fetch|images>"
			echo "	init:   Initialize rrd-datafiles if neeeded"
			echo "	def:    Creates def files required for running"
			echo "	cgi:    Creates cgi-script for use with rrdcgi"
			echo "	fetch:  Fetching data from rrdcollect via netcat"
			echo "	images: Generates images"
			exit 1
			;;
esac
exit 0
